import * as rollup from 'rollup';
import { program, Command } from 'commander';
import path from 'path';
import { fileIsExist } from './utils/file.ts';
import { resolve, commonjs, nodeEntries, alias, esbuild, json } from './app.ts';
import { Chalk } from 'chalk';
const chalk = new Chalk({ level: 3 });
const green = chalk.green;
const blue = chalk.blue;
export async function build(rollupConfig: rollup.RollupOptions) {
  const bundle = await rollup.rollup(rollupConfig);
  const startTime = Date.now();
  const output = rollupConfig.output as rollup.OutputOptions;
  const dir = output?.dir || 'dist';
  const entry = process.cwd();

  const { output: outputFiles } = await bundle.write(output);
  outputFiles.forEach((file) => {
    let filePath = path.resolve(entry, dir, file.fileName);
    // @ts-ignore
    let relativefilePath = path.relative(entry, file.facadeModuleId);
    let relativeOutputPath = path.relative(entry, filePath);
    console.log(blue(`${relativefilePath} → ${relativeOutputPath}`));
  });

  await bundle.close();
  console.log(green(`created ${dir} in ${Date.now() - startTime}ms`));
}

export async function rollupWatch(rollupConfig: rollup.RollupOptions) {
  const watcher = rollup.watch(rollupConfig);
  watcher.on('event', (event) => {
    if (event.code === 'START') {
      console.log(chalk.bold('Rebuilding...'));
    } else if (event.code === 'BUNDLE_END') {
      console.log(chalk.green(`Bundled in ${event.duration}ms.`));
    } else if (event.code === 'END') {
      console.log(chalk.whiteBright('Watching for changes...'));
    } else if (event.code === 'ERROR') {
      console.error(chalk.red('Error:', event.error));
    }
  });
}

// 将多个子命令加入主程序中
program.name('system-apps').description('A CLI tool with envison backend services').version('0.0.1');


const defaultConfig = {
  input: 'src/index.ts',
  output: {
    file: 'dist/index.mjs',
    chunkFileNames: '[name]-[hash].mjs',
    format: 'esm',
  },
  plugins: [
    // @ts-ignore
    alias({ entries: [{ find: '@', replacement: path.resolve('src') }, ...nodeEntries] }),
    // @ts-ignore
    resolve(),
    // @ts-ignore
    commonjs(),
    esbuild({
      target: 'node22', // 目标为 Node.js 14
      minify: false, // 启用代码压缩
      tsconfig: 'tsconfig.json',
    }),
    // @ts-ignore
    json(),
  ],
};

const runBuild = new Command('build')
  .option('-c --config <config>', 'config path')
  .option('-w --watch ', 'watch ')
  .description('Build the project')
  .action(async (options) => {
    const { config, watch } = options;
    const runPath = process.cwd();
    const configRelativePath = config || `rollup.config.mjs`;
    const configPath = path.resolve(runPath, configRelativePath);
    let configs = [];
    if (fileIsExist(configPath)) {
      const config = await import(configPath);
      if (!config.default) {
        throw new Error('config file must have a default export');
      }
      configs = Array.isArray(config.default) ? config.default : [config.default];
    } else {
      configs.push(defaultConfig);
    }
    if (watch) {
      configs = configs.map((config: rollup.RollupOptions) => {
        config.watch = {
          buildDelay: 100,
          clearScreen: false,
          include: 'src/**',
          ...config.watch,
        };
        return config;
      });
      for (const config of configs) {
        await rollupWatch(config);
      }
    } else {
      for (const config of configs) {
        await build(config);
      }
    }
  });

program.addCommand(runBuild);

export { program, Command };
